#include "Prim_and_Kruskal_Algorithms.h"
#include "Undirected_Graph.h"

#include <iostream>
using std::cout;
using std::cin;
using std::endl;

#include <cstdlib>
using std::rand;
using std::srand;

#include <ctime>
using std::time;

//Global Variables and Settings for the program:

const int numIterations = 1; //How many times the Prim function will run.
const int numNodes = 10;     //The number of nodes in each graph.
const int numEdges = 9;      //The number of edges for each graph.
const int sRandWeight = 1;   //The "start" range of the weight of each edge in the graph.
const int eRandWeight = 100; //The "end" range of the weight of each edge in the graph.

void PKA::runAlgorithms()    //Runs the Algorithms.
{
    	srand( time(0) );    //Seeding the random function.

	cout << "------------------------------" << endl;

        //Calling the Functions:
        cout << "\nRunning the Prim's Algorithms:\nPlease wait till the completion of the execution time" << endl;

        //Start the clock for Prim's Algorithm:
        clock_t start, finish;
    	start = clock();

    	for(int iter1 = 1; iter1 <= numIterations; ++iter1)
    	{
        	prim(); //Running the prim's Algorithm
        }

       	//Stop the clock for Prim and print the results:
	finish = clock();
    	cout << "\n\tThe execution time of Prim's Algorithm:\t" << ((double)(finish - start) / CLOCKS_PER_SEC) << " s";
}

void PKA::prim()
{
	//=============================Generating A Random Graph=======================
	//=============================================================================
	//=============================================================================

	//Randomizing Values:

	int randStartNode = rand() % 10;      //Generation a random start node.
	int randEndNode = rand() % 10;        //Generating a random end node.
	int randWeight; 		      //Random weight for the edge.

	while(randEndNode == randStartNode)   //Checking if both randomized nodes are equal.
	{
		randEndNode = (rand() % 10);
	}

	//===============================================

	Graph myGraph(numNodes);              //Create a new Graph.

	for(int i = 0; i < numEdges; ++i)
	{
		//Generating a random weight:
		randWeight =  sRandWeight + rand() % eRandWeight;

		//Inserting a new Edge:
		myGraph.insertEdge(randStartNode, randEndNode, randWeight);
	}


	//=============================================================================
	//=============================================================================
	//=============================================================================

	int currentNode = 0;           //The current Node being under investigation.
	int adjCounter = NULL;         //How many adjacent nodes do we have for the current node.
	int minDistance = NULL;        
	int minIndex = 0;

	myGraph.nodes_List[0]->distance = 0;   //Indicate the start node.
	myGraph.nodes_List[0]->visited = 1;    //The starting node is already considered as a visited node.

	for(int i = 0; i < numNodes - 1; i++) 
	{
		//Determine how many adjacent nodes there are for the current node:
		adjCounter = myGraph.nodes_List[currentNode]->adj.size();  

		if(adjCounter > 0) //If there is more than one adjacent node to the current node:
		{
			minDistance = myGraph.nodes_List[currentNode]->adj.at(0)->w;
			minIndex = 0;
		}

		for(int counter = 0; adjCounter > 0; adjCounter--, counter++)
		{
			if(myGraph.nodes_List[currentNode]->adj[counter]->end_Node->visited == false)
			{
				if(myGraph.nodes_List[currentNode]->distance > myGraph.nodes_List[currentNode]->adj[counter]->w)
				{
					myGraph.nodes_List[currentNode]->distance = myGraph.nodes_List[currentNode]->adj[counter]->w;
				}

				if(minDistance > myGraph.nodes_List[currentNode]->adj[counter]->w)
				{
					minDistance = myGraph.nodes_List[currentNode]->adj[counter]->w;
					minIndex = counter;
				}
			}
		}

                //=========================The Problem is in the following two lines====================
                //======================================================================================
		//======================================================================================
		
		//Switching to the next node that we have just visited:
		myGraph.nodes_List[currentNode]->adj.at(minIndex)->end_Node->visited = 1;
		
		//Mark the current node as visited:
		currentNode = myGraph.nodes_List[currentNode]->adj.at(minIndex)->start_Node->id;
		
		//======================================================================================
		//======================================================================================
		//======================================================================================
	}
}
